@@ -1,4 +1,5 @@
 import { connect } from 'react-redux';
+import { defineMessages, injectIntl } from 'react-intl';
 import ImmutablePropTypes from 'react-immutable-proptypes';
 import React from 'react';
 
@@ -9,22 +10,43 @@ import { getCurrentStack,
 import { getAvailableNodes, getUnassignedAvailableNodes } from '../../selectors/nodes';
 import { getEnvironmentConfigurationSummary } from '../../selectors/environmentConfiguration';
 import { getCurrentPlan } from '../../selectors/plans';
-import { ConfigurePlanStep } from './ConfigurePlanStep';
+import { getRoles } from '../../selectors/roles';
+import ConfigurePlanStep from './ConfigurePlanStep';
 import CurrentPlanActions from '../../actions/CurrentPlanActions';
 import { DeploymentPlanStep } from './DeploymentPlanStep';
-import { DeployStep } from './DeployStep';
+import DeployStep from './DeployStep';
 import EnvironmentConfigurationActions from '../../actions/EnvironmentConfigurationActions';
-import { HardwareStep } from './HardwareStep';
+import HardwareStep from './HardwareStep';
 import PlansDropdown from './PlansDropdown';
 import NodesActions from '../../actions/NodesActions';
 import NoPlans from './NoPlans';
 import NotificationActions from '../../actions/NotificationActions';
 import PlanActions from '../../actions/PlansActions';
 import StacksActions from '../../actions/StacksActions';
-import { RolesStep } from './RolesStep';
+import stackStates from '../../constants/StacksConstants';
+import RolesStep from './RolesStep';
 import RolesActions from '../../actions/RolesActions';
 import ValidationsActions from '../../actions/ValidationsActions';
 
+const messages = defineMessages({
+  hardwareStepHeader: {
+    id: 'DeploymentPlan.hardwareStepHeader',
+    defaultMessage: 'Prepare Hardware'
+  },
+  configureRolesStepHeader: {
+    id: 'DeploymentPlan.configureRolesStepHeader',
+    defaultMessage: 'Configure Roles and Assign Nodes'
+  },
+  deploymentConfigurationStepHeader: {
+    id: 'DeploymentPlan.deploymentConfigurationStepHeader',
+    defaultMessage: 'Specify Deployment Configuration'
+  },
+  deployStepHeader: {
+    id: 'DeploymentPlan.deployStepHeader',
+    defaultMessage: 'Deploy'
+  }
+});
+
 class DeploymentPlan extends React.Component {
   componentDidMount() {
     this.props.fetchStacks();
@@ -32,6 +54,7 @@ class DeploymentPlan extends React.Component {
 
   componentWillReceiveProps(nextProps) {
     if (!nextProps.stacksLoaded) { this.props.fetchStacks(); }
+    this.postDeploymentValidationsCheck(nextProps.currentStack);
     this.pollCurrentStack(nextProps.currentStack);
   }
 
@@ -51,12 +74,27 @@ class DeploymentPlan extends React.Component {
     }
   }
 
+  postDeploymentValidationsCheck(nextStack) {
+    const { currentStack, currentPlan } = this.props;
+    const progressStates = [stackStates.UPDATE_IN_PROGRESS, stackStates.CREATE_IN_PROGRESS];
+    const successStates = [stackStates.UPDATE_COMPLETE, stackStates.CREATE_COMPLETE];
+    if (currentStack
+        && nextStack
+        && progressStates.includes(currentStack.stack_status)
+        && successStates.includes(nextStack.stack_status)) {
+      this.props.runPostDeploymentValidations(currentPlan.name);
+    }
+  }
+
   render() {
+    const { formatMessage } = this.props.intl;
     let children;
+    const currentPlanName = this.props.hasPlans ? this.props.currentPlan.name : undefined;
+
     // Render children only when current plan is already selected
-    if (this.props.children && this.props.currentPlan.name) {
+    if (this.props.children && currentPlanName) {
       children = React.cloneElement(this.props.children,
-                                    {currentPlanName: this.props.currentPlan.name,
+                                    {currentPlanName: currentPlanName,
                                      parentPath: '/' + this.props.route.path});
     }
 
@@ -66,38 +104,38 @@ class DeploymentPlan extends React.Component {
           <div className="col-sm-12">
             <div className="page-header page-header-bleed-right">
               <h1>
-                {this.props.currentPlan.name}
-                <PlansDropdown currentPlanName={this.props.currentPlan.name}
+                {currentPlanName}
+                <PlansDropdown currentPlanName={currentPlanName}
                                plans={this.props.inactivePlans}
                                choosePlan={this.props.choosePlan}/>
               </h1>
             </div>
             <ol className="deployment-step-list">
-              <DeploymentPlanStep title="Prepare Hardware"
-                                  disabled={this.props.currentStackDeploymentInProgress}>
-                <HardwareStep />
-              </DeploymentPlanStep>
-              <DeploymentPlanStep title="Specify Deployment Configuration"
-                                  disabled={this.props.currentStackDeploymentInProgress}>
+            <DeploymentPlanStep title={formatMessage(messages.hardwareStepHeader)}
+                                disabled={this.props.currentStackDeploymentInProgress}>
+              <HardwareStep />
+            </DeploymentPlanStep>
+            <DeploymentPlanStep title={formatMessage(messages.deploymentConfigurationStepHeader)}
+                                disabled={this.props.currentStackDeploymentInProgress}>
                 <ConfigurePlanStep
                   fetchEnvironmentConfiguration={this.props.fetchEnvironmentConfiguration}
                   summary={this.props.environmentConfigurationSummary}
-                  planName={this.props.currentPlan.name}
+                  planName={currentPlanName}
                   isFetching={this.props.isFetchingEnvironmentConfiguration}
                   loaded={this.props.environmentConfigurationLoaded}/>
-              </DeploymentPlanStep>
-              <DeploymentPlanStep title="Configure Roles and Assign Nodes"
-                                  disabled={this.props.currentStackDeploymentInProgress}>
+            </DeploymentPlanStep>
+            <DeploymentPlanStep title={formatMessage(messages.configureRolesStepHeader)}
+                                disabled={this.props.currentStackDeploymentInProgress}>
                 <RolesStep availableNodes={this.props.availableNodes}
                            fetchNodes={this.props.fetchNodes}
-                           fetchRoles={this.props.fetchRoles}
+                           fetchRoles={this.props.fetchRoles.bind(this, currentPlanName)}
                            isFetchingNodes={this.props.isFetchingNodes}
                            isFetchingRoles={this.props.isFetchingRoles}
                            roles={this.props.roles}
                            rolesLoaded={this.props.rolesLoaded}
                            unassignedAvailableNodes={this.props.unassignedAvailableNodes}/>
               </DeploymentPlanStep>
-              <DeploymentPlanStep title="Deploy">
+            <DeploymentPlanStep title={formatMessage(messages.deployStepHeader)}>
                 <DeployStep
                   currentPlan={this.props.currentPlan}
                   currentStack={this.props.currentStack}
@@ -109,8 +147,6 @@ class DeploymentPlan extends React.Component {
                   fetchStackEnvironment={this.props.fetchStackEnvironment}
                   fetchStackResource={this.props.fetchStackResource}
                   isRequestingStackDelete={this.props.isRequestingStackDelete}
-                  runPostDeploymentValidations={
-                    this.props.runPostDeploymentValidations.bind(this.props.currentPlan.name)}
                   stacksLoaded={this.props.stacksLoaded}/>
               </DeploymentPlanStep>
             </ol>
@@ -149,6 +185,7 @@ DeploymentPlan.propTypes = {
   fetchStacks: React.PropTypes.func,
   hasPlans: React.PropTypes.bool,
   inactivePlans: ImmutablePropTypes.map,
+  intl: React.PropTypes.object,
   isFetchingEnvironmentConfiguration: React.PropTypes.bool,
   isFetchingNodes: React.PropTypes.bool,
   isFetchingRoles: React.PropTypes.bool,
@@ -179,7 +216,7 @@ export function mapStateToProps(state) {
     hasPlans: !state.plans.get('all').isEmpty(),
     inactivePlans: getAllPlansButCurrent(state),
     availableNodes: getAvailableNodes(state),
-    roles: state.roles.get('roles'),
+    roles: getRoles(state),
     rolesLoaded: state.roles.get('loaded'),
     stacksLoaded: state.stacks.get('isLoaded'),
     unassignedAvailableNodes: getUnassignedAvailableNodes(state)
@@ -198,7 +235,7 @@ function mapDispatchToProps(dispatch) {
       dispatch(EnvironmentConfigurationActions.fetchEnvironmentConfiguration(planName, parentPath));
     },
     fetchNodes: () => dispatch(NodesActions.fetchNodes()),
-    fetchRoles: () => dispatch(RolesActions.fetchRoles()),
+    fetchRoles: planName => dispatch(RolesActions.fetchRoles(planName)),
     fetchStackResources: (stack) =>
       dispatch(StacksActions.fetchResources(stack.stack_name, stack.id)),
     fetchStackResource: (stack, resourceName) =>
@@ -211,4 +248,4 @@ function mapDispatchToProps(dispatch) {
   };
 }
 
-export default connect(mapStateToProps, mapDispatchToProps)(DeploymentPlan);
+export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(DeploymentPlan));
